@@ -552,13 +552,19 @@ get_mail_from_control (LPDISPATCH ctrl, bool *none_selected)
         }
     }
 
-  auto ret = Mail::getMailForUUID (uid);
+  auto ret = Mail::getMailForUUID(uid);
+  if (ret==NULL)
+  {
+    auto retV = Mail::searchMailsByUUID(uid);
+    if (retV.empty())
+      {
+        log_error ("%s:%s: Failed to find mail %p in map.",
+                  SRCNAME, __func__, mailitem);
+      }
+    ret = retV.front();
+  }
   xfree (uid);
-  if (!ret)
-    {
-      log_error ("%s:%s: Failed to find mail %p in map.",
-                 SRCNAME, __func__, mailitem);
-    }
+
   /* This release may have killed the Mail object we obtained earlier
    * if it was just a leftover mail in the ribbon. */
   gpgol_release (mailitem);
@@ -840,7 +846,7 @@ HRESULT get_is_crypto_mail (LPDISPATCH ctrl, VARIANT *result)
   MY_MAIL_GETTER
 
   result->vt = VT_BOOL | VT_BYREF;
-  result->pboolVal = mail && (mail->isSigned () || mail->isEncrypted ()) ?
+  result->pboolVal = mail && ((mail->isSigned () &&!mail->isEncrypted()) || mail->realyDecryptedSuccessfully ()) ?
                           &var_true : &var_false;
 
   TRACEPOINT;
